@@ -18,6 +18,7 @@ from unimatrix.exceptions import CanonicalException
 
 from .apiroute import APIRoute
 from .exceptions import BaseRedirectingException
+from .exceptions import HTTPException
 from .exceptions import TryAgain
 from .exceptions import UnprocessableEntity
 from .exceptions import UpstreamServiceNotAvailable
@@ -26,6 +27,7 @@ from .ext import ioc
 from .types import IRequestHandler
 from .types import IRouteable
 from .types import Request
+from .utils import retry
 
 
 class Application(fastapi.FastAPI):
@@ -108,6 +110,7 @@ class Application(fastapi.FastAPI):
             RequestValidationError: self.canonical_exception,
             TimeoutError: self.canonical_exception,
             RemoteProtocolError: self.canonical_exception,
+            HTTPException: self.canonical_exception
         })
         super().__init__(*args, **kwargs)
         self.router.route_class = APIRoute
@@ -180,10 +183,7 @@ class Application(fastapi.FastAPI):
     async def _boot(self): # pragma: no cover
         self.logger.info("Booting up ASGI application")
         await unimatrix.runtime.on('boot')
-        await self.keychain
-        if self.jwks_tags:
-            keychain = self.keychain.tagged(self.jwks_tags)
-            self.jwks = keychain.as_jwks(private=False)
+        await self.setup_keychain()
         await ioc.on_boot(self)
         await self.boot()
 
@@ -238,6 +238,8 @@ class Application(fastapi.FastAPI):
         elif isinstance(exception, (asyncio.TimeoutError, TimeoutError)):
             #count = int(request.headers.get('X-Retry') or 0)
             return await self.canonical_exception(request, TryAgain(30))
+        elif isinstance(exception, HTTPException):
+            return await exception.as_response()
         elif isinstance(exception, CanonicalException):
             status_code = getattr(exception, 'http_status_code', None) or 500
             if self.log_exceptions:
@@ -297,6 +299,13 @@ class Application(fastapi.FastAPI):
         await self.teardown()
         await ioc.on_shutdown(self)
 
+    @retry(2, 0.01)
+    async def setup_keychain(self) -> None:
+        await self.keychain
+        if self.jwks_tags:
+            keychain = self.keychain.tagged(self.jwks_tags)
+            self.jwks = keychain.as_jwks(private=False)
+
     async def teardown(self): # pragma: no cover
         """Invoked when the :class:`cbra.Application` is tearing down."""
         pass
