@@ -22,6 +22,15 @@ from cbra.types import IRequestHandler
 from cbra.utils import DeferredException
 
 
+
+class HTTPException(Exception):
+    """Mixin class for exceptions that can be converted to a response."""
+    __module__: str = 'cbra.exceptions'
+
+    async def as_response(self) -> fastapi.responses.Response:
+        raise NotImplementedError
+
+
 class IExceptionSpec(pydantic.BaseModel):
     code: str = pydantic.Field(
         default=...,
@@ -598,6 +607,14 @@ class UnprocessableEntity(CanonicalException):
             instance = cls(errors=errors)
         return instance
 
+    def as_dict(self) -> dict[str, Any]:
+        dto = super().as_dict()
+        if self.params.get('errors'):
+            dto['spec']['fieldErrors'] = {
+                str.join('.', k): list(v) for k, v in self.params['errors'].items()
+            }
+        return dto
+
 
 class Unserializable(ValueError):
     pass
