@@ -8,6 +8,7 @@ from cbra.resource import Resource
 from . import exceptions
 from . import params
 from . import types
+from .authorization import Authorization
 from .authorizeendpoint import AuthorizationEndpoint
 from .authorizationserver import AuthorizationServer
 from .authorizationrequestclient import AuthorizationRequestClient
@@ -36,6 +37,7 @@ __all__: list[str] = [
     'params',
     'scope',
     'types',
+    'Authorization',
     'AuthorizationEndpoint',
     'AuthorizationServer',
     'AuthorizationRequestClient',
