@@ -1,11 +1,14 @@
 """Declares :class:`AuthorizationEndpoint`."""
+import datetime
 import typing
 import urllib.parse
+from typing import Any
 
 import fastapi
 
 from cbra.types import IPrincipal
 from ckms.jose import PayloadCodec
+from .authorization import Authorization
 from .authorizationrequestclient import AuthorizationRequestClient
 from .endpoint import Endpoint
 from .exceptions import CrossOriginNotAllowed
@@ -21,6 +24,8 @@ from .params import SubjectRepository
 from .params import TransientStorage
 from .params import UpstreamProvider
 from .requestobjectdecoder import RequestObjectDecoder
+from .types import AuthorizationException
+from .types import AuthorizationIdentifier
 from .types import AuthorizationRequest
 from .types import AuthorizationRequestParameters
 from .types import IAuthorizationServer
@@ -40,6 +45,7 @@ class AuthorizationEndpoint(Endpoint, IAuthorizeEndpoint):
     codec: PayloadCodec = ServerCodec
     summary: str = "Authorization Endpoint"
     decoder: RequestObjectDecoder
+    denied_url: str
     description: str = (
         "The **Authorization Endpoint** is used to interact with the resource "
         "owner and obtain an authorization grant."
@@ -55,6 +61,7 @@ class AuthorizationEndpoint(Endpoint, IAuthorizeEndpoint):
         "Communicates the allowed methods and CORS options for "
         "the **Authorization Request** endpoint."
     )
+    prompt_url: str
     provider: IUpstreamProvider | None
 
     def __init__(
@@ -86,14 +93,28 @@ class AuthorizationEndpoint(Endpoint, IAuthorizeEndpoint):
     async def add_to_client(
         self,
         client: IClient,
-        principal: IPrincipal
+        principal: IPrincipal,
+        dto: AuthorizationRequest
     ) -> ISubject:
         """Add the principal given its subject identifier to the client. This
         method is invoked when handling an authorization request for a client
         with which a subject has never authenticated before.
         """
+        await self.onboard(client, principal, dto)
         return await self.subjects.persist(client, principal)
 
+    async def onboard(
+        self,
+        client: Any,
+        principal: Any,
+        dto: AuthorizationRequest
+    ) -> None:
+        """Onboard the Subject with the Client. The default implementation
+        does nothing; subclasses may override this method in order to provide
+        customized onboarding policies.
+        """
+        pass
+
     async def create_redirect(
         self,
         client: IClient,
@@ -122,13 +143,32 @@ class AuthorizationEndpoint(Endpoint, IAuthorizeEndpoint):
 
     async def create_upstream_redirect(
         self,
-        dto: AuthorizationRequestParameters
+        dto: AuthorizationRequest
     ) -> str:
         """Return a string containing the redirection endpoint at the
         upstream identity provider.
         """
         assert self.provider is not None # nosec
-        return await self.provider.create_redirect(self.request, dto)
+        return await self.provider.create_redirect(self.request, dto) # type: ignore
+
+    async def enforce_policy(
+        self,
+        dto: AuthorizationRequest,
+        client: IClient,
+        subject: ISubject
+    ) -> None:
+        """Enforces the access policy for the given client and subject."""
+        failures: list[AuthorizationException] = []
+        if dto.claims:
+            claims = self.openid.parse_claims(dto.claims)
+            errors = await self.openid.enforce_claims(
+                dto, client, subject, claims.id_token
+            )
+            failures.extend(errors) # type: ignore
+        for error in failures:
+            error.log(self.logger)
+        if failures:
+            self.deny(dto, failures[0])
 
     async def enforce_cors_policy( # type: ignore
         self,
@@ -165,9 +205,14 @@ class AuthorizationEndpoint(Endpoint, IAuthorizeEndpoint):
                 client=client,
                 subject_id=principal.sub,
                 force_public=True
-            ) or await self.add_to_client(client, principal)
+            ) or await self.add_to_client(client, principal, dto)
         assert subject is not None
         if subject and principal and (subject.sub != principal.sub):
+            self.logger.critical(
+                "Principal does not authenticate the Subject "
+                "(principal: %s, subject: %s)",
+                self.principal.sub, subject.sub
+            )
             raise Error(
                 error="invalid_request",
                 error_description=(
@@ -196,7 +241,6 @@ class AuthorizationEndpoint(Endpoint, IAuthorizeEndpoint):
                 )
             )
         if params.is_object():
-            raise Exception
             # A Request Object as specified in RFC 9101
             assert params.request is not None
             claims = await self.decoder.decode_request_object(params.request)
@@ -215,23 +259,79 @@ class AuthorizationEndpoint(Endpoint, IAuthorizeEndpoint):
             )
 
         assert params.request_id is not None # nosec
-        return await self.storage.get_authorization_request(params.request_id)
+        request = await self.storage.get_authorization_request(params.request_id)
+        if request.client_id != client.client_id:
+            self.logger.critical(
+                "Authorization request (id: %s) was not issued by client %s",
+                request.request_id, client.client_id
+            )
+            raise Error(
+                error="invalid_request",
+                error_description="The authorization request was not valid."
+            )
+        return request
 
     async def handle(
         self,
         client: IClient = AuthorizationRequestClient,
         params: AuthorizationRequestParameters = fastapi.Depends()
-    ):
+    ) -> fastapi.responses.Response:
+        """Initiates the **Authorization Code Flow**."""
+        request: None | AuthorizationRequest = None
+        try:
+            request = await self.resolve(client, params)
+            return await self.get_redirect_response(client, request)
+        except Exception as e:
+            return await self.on_fatal_exception(client, request, e)
+
+    async def on_fatal_exception(
+        self,
+        client: Any,
+        request: Any | None,
+        e: Exception
+    ) -> fastapi.responses.Response:
+        """Hook that is invoked when a fatal exception occurs during the authorization
+        flow.
+        """
+        raise
+
+    async def get_authorization(
+        self,
+        client_id: int | str,
+        sub: int | str
+    ) -> Authorization:
+        """Return the :class:`~cbra.ext.oauth2.Authorization` that is
+        granted to the Client by the Resource Owner. If the Client did
+        not obtain any grant previously, return the object without any
+        scopes.
+        """
+        key = AuthorizationIdentifier(client_id=client_id, sub=sub)
+        authorization = await self.storage.get(key)
+        if authorization is None:
+            authorization = Authorization.fromkey(key)
+        assert isinstance(authorization, Authorization) # nosec
+        return authorization
+
+    async def get_redirect_response(
+        self,
+        client: IClient,
+        dto: AuthorizationRequest
+    ) -> fastapi.responses.Response:
         """Initiates the **Authorization Code Flow**."""
-        dto = await self.resolve(client, params)
         if not self.principal.is_authenticated():
-            return await self.on_login_required(dto)
+            await self.on_login_required(dto)
+            assert False # nosec
         subject = await self.get_subject(
             client=client,
             principal=self.principal,
             dto=dto
         )
+        assert self.principal.sub is not None # nosec
         assert subject is not None # nosec
+        authorization = await self.get_authorization(
+            client.client_id,
+            self.principal.sub
+        )
 
         # Check if the request parameters are valid.
         await self.validate_request(
@@ -252,9 +352,24 @@ class AuthorizationEndpoint(Endpoint, IAuthorizeEndpoint):
         redirect_uri = await self.create_redirect(client, dto=dto)
         dto.auth_time = self.principal.get_auth_time()
         assert dto.sub is not None
-        await self.validate(dto, client, subject, self.principal)
-        await self.on_success(dto, subject)
-        await self.persist(dto)
+        try:
+            await self.validate(dto, client, subject, self.principal)
+            await self.enforce_policy(
+                dto=dto,
+                client=client,
+                subject=subject
+            )
+            await self.on_success(dto, client, subject)
+
+            # TODO: The scope should be accepted out of band.
+            now = datetime.datetime.utcnow()
+            for scope in dto.scope:
+                authorization.authorize(scope, now=now)
+            if dto.allows_refresh():
+                authorization.enable_refresh()
+        finally:
+            await self.persist(dto)
+            await self.persist(authorization)
         return fastapi.responses.RedirectResponse(
             url=redirect_uri,
             status_code=303
@@ -262,8 +377,9 @@ class AuthorizationEndpoint(Endpoint, IAuthorizeEndpoint):
 
     async def on_success(
         self,
-        dto: AuthorizationRequest,
-        subject: ISubject
+        dto: Any,
+        client: Any,
+        subject: Any
     ) -> None:
         """Invoked when the authorization request can successfully
         proceed.
@@ -312,6 +428,20 @@ class AuthorizationEndpoint(Endpoint, IAuthorizeEndpoint):
                 }
             )
 
+    def deny(
+        self,
+        dto: AuthorizationRequest,
+        error: AuthorizationException
+    ) -> typing.NoReturn:
+        """Denies the authorization request. If the request is able to interact
+        with the end-user, prompt the user displaying the reason. Otherwise,
+        display a generic message.
+        """
+        if dto.can_interact():
+            assert self.login_url is not None # nosec
+            error.raise_for_user(self.denied_url, dto.redirect_uri)
+        raise NotImplementedError("Can not interact")
+
     def interact(self, dto: AuthorizationRequest) -> typing.NoReturn:
         """Interact with the end-user."""
         if not self.login_url:
@@ -339,13 +469,14 @@ class AuthorizationEndpoint(Endpoint, IAuthorizeEndpoint):
 
     async def persist(
         self,
-        dto: AuthorizationRequest
+        dto: AuthorizationRequest | Authorization
     ) -> None:
         """Inspect the request parameters and persist the appropriate
         objects to the transient storage.
         """
         await self.storage.persist(dto)
-        if dto.needs_authorization_code():
+        if isinstance(dto, AuthorizationRequest)\
+        and dto.needs_authorization_code():
             await self.storage.persist(dto.get_authorization_code())
 
     def redirect_consent(self, dto: AuthorizationRequest) -> fastapi.Response:
