@@ -1,5 +1,6 @@
 # type: ignore
 """Declares :class:`ClientConfig`."""
+import datetime
 import logging
 import secrets
 import typing
@@ -7,6 +8,7 @@ from typing import Any, Literal
 
 import httpx
 import pydantic
+from cbra.exceptions import UpstreamServiceNotAvailable
 from ckms.core.models import JSONWebSignature
 from ckms.types import IKeychain
 from ckms.types import JSONWebKey
@@ -25,10 +27,12 @@ from .exceptions import RedirectForbidden
 from .types import ISubject
 from .types import GrantType
 from .types import RedirectURL
+from .types import RefreshTokenPolicy
 from .types import ResponseType
 
 
 logger: logging.Logger = logging.getLogger('uvicorn')
+JWKSLookupFailed: UpstreamServiceNotAvailable = UpstreamServiceNotAvailable(5)
 
 
 class ClientConfig(BaseClient):
@@ -60,6 +64,10 @@ class ClientConfig(BaseClient):
 
     redirect_uris: list[RedirectURL] = []
 
+    refresh_token_policy: RefreshTokenPolicy = RefreshTokenPolicy.rolling
+
+    refresh_token_ttl: int = 0
+
     response_modes: set[str] = set()
 
     response_types: set[ResponseType] = set()
@@ -207,29 +215,92 @@ class ClientConfig(BaseClient):
             url = None
         return url
 
+    def get_refresh_token_expires(self, now: int) -> int:
+        """Return a :class:`datetime.datetime` instance indicating the
+        Time-To-Live (TTL) of a refresh token.
+        """
+        return now + self.refresh_token_ttl
+
     def get_subject_id(self, subject: ISubject) -> int:
         """Return the subject identifier to add to an access token or ID
         token.
         """
         return subject.get_identifier(public=self.subject_type == "public")
 
+    async def create_refresh_token(
+        self,
+        codec: PayloadCodec,
+        subject: ISubject,
+        authorization_id: int,
+        token_id: int,
+        expires: datetime.datetime | int | None
+    ) -> str:
+        now = current_timestamp()
+        if isinstance(expires, datetime.datetime):
+            expires = int(expires.timestamp())
+        exp = expires or self.get_refresh_token_expires(now)
+        return await codec.encode(
+            payload={
+                'authorization_id': authorization_id,
+                'client_id': self.client_id,
+                'exp': exp,
+                'iat': now,
+                'iss': token_id,
+                'jti': secrets.token_urlsafe(24),
+                'nbf': now
+            },
+            content_type="rt+jwt"
+        )
+
     async def import_jwks(self) -> JSONWebKeySet:
         """Retrieve the JSON Web Key Set (JWKS) that is specified by the
         :attr:`jwks_uri` attribute.
+
+        Raises:
+            :class:`~cbra.exceptions.UpstreamServiceNotAvaialble`: unable
+                to lookup the JWKS through :attr:`jwks_uri`.
         """
         jwks = self.jwks
         if not jwks.keys and self.jwks_uri is not None:
-            async with httpx.AsyncClient() as client:
-                logger.info("Obtaining client keys from %s", self.jwks_uri[:256])
-                response = await client.get(self.jwks_uri)
-                if response.status_code == 200:
-                    try:
-                        jwks = JSONWebKeySet.parse_obj(response.json())
-                        self.keys = jwks.keys
-                    except pydantic.ValidationError as e:
-                        logger.warning("Unable to parse JWKS from %s", self.jwks_uri[:256])
-                else:
-                    logger.warning("%s returned non-200 response", self.jwks_uri[:256])
+            async with httpx.AsyncClient(timeout=5.0) as client:
+                logger.info(
+                    "Obtaining client (id: %s) keys from %s",
+                    self.client_id, self.jwks_uri
+                )
+                try:
+                    response = await client.get(self.jwks_uri)
+                    response.raise_for_status()
+                except (httpx.RequestError, httpx.HTTPError):
+                    # Occurs when there is a network problem i.e. connection timeout,
+                    # DNS not resolved
+                    logger.critical(
+                        "Unable to lookup client (id: %s) JWKS from %s",
+                        self.client_id, self.jwks_uri
+                    )
+                    raise JWKSLookupFailed
+                try:
+                    jwks = JSONWebKeySet.parse_obj(response.json())
+                    keys = jwks.keys
+                    if not jwks.keys or not [x for x in jwks.keys if x.use == 'sig']:
+                        logger.critical(
+                            "No signing keys in client (id: %s) JWKS retrieved from %s",
+                            self.client_id, self.jwks_uri
+                        )
+                        raise UpstreamServiceNotAvailable(
+                            code="INVALID_JWKS",
+                            message="The JSON Web Key Set (JWKS) did not contain any signing keys."
+                        )
+                    self.keys = jwks.keys
+                except (pydantic.ValidationError, TypeError, ValueError) as e:
+                    logger.critical("Unable to parse JWKS from %s", self.jwks_uri)
+                    raise UpstreamServiceNotAvailable(
+                        code="INVALID_JWKS_FORMAT",
+                        message=(
+                            f"The JSON Web Key Set (JWKS) retrieved from {self.jwks_uri} "
+                            "could not be interpreted as JSON or the schema of the JSON "
+                            "object was invalid."
+                        )
+                    )
         return jwks
 
     def is_first_party(self) -> bool:
