@@ -12,6 +12,7 @@ from .exceptions import Error
 from .exceptions import LoginRequired
 from .params import CurrentServerMetadata
 from .types import ServerMetadata
+from .types.promptexception import PromptException
 
 
 class Endpoint(cbra.Endpoint):
@@ -91,6 +92,13 @@ class Endpoint(cbra.Endpoint):
             error_url=self.error_url
         )
 
+    @cbra.Endpoint.on_exception.register # type: ignore
+    async def on_prompt(
+        self,
+        exception: PromptException
+    ) -> fastapi.Response:
+        return await exception.as_response()
+
     @cbra.Endpoint.on_exception.register # type: ignore
     async def on_login_required(
         self,
