@@ -1,6 +1,7 @@
 """Declares :class:`MemorySubjectRepository`."""
 import typing
 
+from .types import IClient
 from .types import ISubject
 from .types import ISubjectRepository
 
@@ -10,7 +11,7 @@ class MemorySubjectRepository(ISubjectRepository):
     memory for storage. Mainly for testing purposes.
     """
     __module__: str = 'cbra.ext.oauth2'
-    subjects: typing.Dict[typing.Union[int, str], ISubject] = {}
+    subjects: dict[int | str, ISubject] = {}
 
     @classmethod
     def add(cls, subject: ISubject) -> None:
@@ -23,7 +24,7 @@ class MemorySubjectRepository(ISubjectRepository):
         cls.subjects = {}
 
     @classmethod
-    def configure(cls, subjects: typing.Dict[str, ISubject]) -> None:
+    def configure(cls, subjects: dict[str, ISubject]) -> None:
         """Load a mapping of subject identifiers to subjects."""
         cls.subjects.update(subjects) # type: ignore
 
@@ -32,12 +33,13 @@ class MemorySubjectRepository(ISubjectRepository):
 
     async def get(
         self,
-        subject_id: typing.Union[int, str],
-        client_id: typing.Optional[str] = None
+        subject_id: int | str,
+        client: IClient | None = None
     ) -> ISubject:
+        assert client is not None # nosec
         try:
             subject = self.subjects[subject_id]
-            subject.client_id = client_id
+            subject.client_id = client.client_id
             return subject
         except KeyError:
             raise self.SubjectDoesNotExist
\ No newline at end of file
