@@ -47,4 +47,7 @@ class Principal(IPrincipal):
         """
         if isinstance(scope, str):
             scope = {scope}
-        return self.scope >= scope
\ No newline at end of file
+        return self.scope >= scope
+
+    def is_authenticated(self) -> bool:
+        return True
\ No newline at end of file
