@@ -3,6 +3,7 @@ import functools
 import inspect
 import logging
 import typing
+from typing import Any
 
 from ckms.core import Keychain
 from ckms.jose import PayloadCodec
@@ -11,6 +12,7 @@ from ckms.types import Malformed
 
 from cbra.conf import settings
 from cbra.params import ServerKeychain
+from .authorization import Authorization
 from .exceptions import InvalidClient
 from .exceptions import InvalidGrant
 from .exceptions import InvalidScope
@@ -23,6 +25,7 @@ from .params import ServerCodec
 from .params import SubjectRepository
 from .params import TransientStorage
 from .types import AuthorizationCodeGrant
+from .types import AuthorizationIdentifier
 from .types import BaseGrant
 from .types import ClientCredentialsGrant
 from .types import GrantType
@@ -33,9 +36,14 @@ from .types import ISubject
 from .types import ISubjectRepository
 from .types import ITokenIssuer
 from .types import JWTBearerAssertionGrant
+from .types import NullRefreshToken
+from .types import RefreshToken
+from .types import RefreshTokenGrant
+from .types import RefreshTokenIdentifier
 from .types import ScopedGrant
 from .types import SessionGrant
 from .types import ServerMetadata
+from .types import TokenException
 from .types import TokenResponse
 from .utils import classproperty # type: ignore
 
@@ -62,6 +70,7 @@ class TokenIssuer(ITokenIssuer):
     def grant_types_supported(cls) -> typing.List[str]:
         return [
             GrantType.authorization_code.value,
+            GrantType.refresh_token.value,
             GrantType.client_credentials.value,
             GrantType.jwt_bearer.value
         ]
@@ -98,7 +107,83 @@ class TokenIssuer(ITokenIssuer):
         """Dispatches a request to the **Token Endpoint** to the
         appropriate handler for its ``grant_type``.
         """
-        raise NotImplementedError(dto.grant_type)
+        self.logger.critical("Invalid grant requested: %s", dto.grant_type.value)
+        raise TokenException(
+            code='invalid_grant',
+            description=(
+                "The server allows but has not implemented the use of the "
+                f"'{dto.grant_type.value}' grant."
+            )
+        )
+
+    @grant.register
+    async def grant_refresh_token(
+        self,
+        grant: RefreshTokenGrant
+    ) -> TokenResponse:
+        """Rotate the existing refresh token and issue a new access token
+        and refresh token.
+        """
+        client = grant.get_client()
+        if not client.allows_audience(self.issuer, grant.resource):
+            raise TokenException(
+                code='invalid_grant',
+                description=(
+                    "The Client does not allow the creation of access tokens "
+                    "for the given resource."
+                )
+            )
+        key = RefreshTokenIdentifier.parse_jwt(client.client_id, grant.refresh_token)
+        token = await self.storage.get(key)
+        if token is None or not await token.verify(grant.refresh_token):
+            if token is not None:
+                self.logger.info(
+                    "Signature validation failure for token (id: %s)",
+                    token.id
+                )
+            raise TokenException(
+                code='invalid_grant',
+                description="The refresh token is invalid, expired or revoked."
+            )
+        assert isinstance(token, RefreshToken) # nosec
+        if not token.allows_scope(grant.scope):
+            raise TokenException(
+                code='invalid_grant',
+                description=(
+                    "The requested scope exceeds the scope that was "
+                    "granted by the Resource Owner."
+                )
+            )
+
+        subject = await self.subjects.get(
+            client=client,
+            subject_id=token.sub,
+            force_public=True
+        )
+        if subject is None:
+            raise TokenException(
+                code='invalid_grant',
+                description="The refresh token is invalid, expired or revoked."
+            )
+
+        result = TokenResponse(
+            access_token=await client.issue_token(
+                codec=self.codec,
+                using=SIGNING_KEY,
+                issuer=self.metadata.issuer,
+                audience=grant.get_audience(),
+                subject=subject,
+                ttl=self.default_ttl,
+                scope=set(grant.scope or []) or set(token.scope)
+            ),
+            id_token=None,
+            state=None,
+            token_type="Bearer",
+            expires_in=self.default_ttl,
+            refresh_token=await token.generate(client, subject, rotate=True)
+        )
+        await self.storage.persist(token)
+        return result
 
     @grant.register
     async def grant_authorization_code(
@@ -107,9 +192,24 @@ class TokenIssuer(ITokenIssuer):
     ) -> TokenResponse:
         client = grant.get_client()
         _, request = await self.storage.get_code(grant.code)
-
-        assert request.is_authenticated() # nosec
+        if not request.is_authenticated():
+            raise TokenException(
+                code='invalid_grant',
+                description=(
+                    "The authorization request was not authorized by the "
+                    "Resource Owner."
+                )
+            )
         subject = await request.get_subject(client, self.subjects)
+        if subject is None:
+            raise TokenException(code='invalid_grant')
+        authorization = await self.storage.get(
+            AuthorizationIdentifier(client.client_id, subject.sub)
+        )
+        if authorization is None:
+            raise TypeError("Authorization must exist.")
+        assert isinstance(authorization, Authorization)
+
         request.validate_grant(client, subject, grant)
         self.validate_grant(
             client=client,
@@ -129,6 +229,11 @@ class TokenIssuer(ITokenIssuer):
             scope=request.scope
         )
         await self.storage.delete_authorization_request(request)
+        rt = NullRefreshToken()
+        if request.wants_refresh_token():
+            rt = authorization.create_refresh_token(request.scope)
+            await self.storage.persist(rt)
+
         return TokenResponse(
             access_token=at,
             token_type="Bearer",
@@ -143,7 +248,7 @@ class TokenIssuer(ITokenIssuer):
                 access_token=at
             ),
             state=request.state,
-            refresh_token=None
+            refresh_token=await rt.generate(client, subject)
         )
 
     @grant.register
@@ -362,9 +467,15 @@ class TokenIssuer(ITokenIssuer):
             force_public=True
         )
         if subject is None:
+            # If subject is None, then it is not onboarded with the client
+            # yet.
             self.logger.info("Subject %s not found.", data.sub)
-            self.fail("The Subject specified by the session does not exist.")
-            assert False # nosec
+            subject = await self.issue_pairwise_identifier(self.subjects, client, data.sub)
+            assert subject is not None # nosec
+            self.logger.info(
+                "Onboarded Subject (sub: %s, client: %s, ppid: %s)",
+                data.sub, client.client_id, subject.get_identifier(public=False)
+            )
         self.validate_grant(client=client, subject=subject, grant=dto)
         self.validate_scope(client=client, subject=subject, grant=dto)
         return TokenResponse(
@@ -416,4 +527,23 @@ class TokenIssuer(ITokenIssuer):
         grant.validate_grant(client, subject)
 
     def fail(self, description: str) -> typing.NoReturn:
-        raise InvalidGrant(error_description=description)
\ No newline at end of file
+        raise InvalidGrant(error_description=description)
+
+    async def issue_pairwise_identifier(
+        self,
+        subjects: ISubjectRepository,
+        client: Any,
+        subject_id: Any
+    ) -> ISubject:
+        """Issue a pairwise identifier for the given :class:`~cbra.ext.oauth2.types.IClient`
+        instance, and return a :class:`~cbra.ext.oauth2.types.ISubject` instance.
+
+        Args:
+            client (:class:`~cbra.ext.oauth2.types.IClient`): the client for which a token
+                is requested.
+            subject_id (any): the **public** identifier of a Subject.
+
+        Returns:
+            :class:`~cbra.ext.oauth2.types.ISubject`
+        """
+        raise NotImplementedError
\ No newline at end of file
