@@ -15,6 +15,7 @@ from .tokenissuer import TokenIssuer
 from .tokenrequest import TokenRequest
 from .types import BaseGrant
 from .types import IAuthorizationServer
+from .types import TokenException
 from .types import TokenResponse
 
 
@@ -96,4 +97,16 @@ class TokenRequestHandler(Endpoint):
                 error="server_error",
                 error_description=exception.message
             )
-        raise
\ No newline at end of file
+        elif isinstance(exception, TokenException):
+            raise
+        self.logger.exception("Caught fatal %s", type(exception).__name__)
+        return fastapi.responses.JSONResponse(
+            status_code=400,
+            content={
+                'error': 'server_error',
+                'error_description': (
+                    "The authorization server encountered an error condition "
+                    "from which it was unable to recover."
+                )
+            }
+        )
\ No newline at end of file
