@@ -12,6 +12,8 @@ from ..exceptions import ForbiddenScope
 from ..exceptions import InvalidGrant
 from ..exceptions import UnsupportedResponseMode
 from ..exceptions import UnsupportedResponseType
+from .accesstype import AccessType
+from .authorizationidentifier import AuthorizationIdentifier
 from .authorizationcode import AuthorizationCode
 from .authorizationcodegrant import AuthorizationCodeGrant
 from .authorizationrequestparameters import AuthorizationRequestParameters
@@ -23,6 +25,7 @@ from .redirecturl import RedirectURL
 from .responsetype import ResponseType
 from .jar import JAR
 from .servermetadata import ServerMetadata
+from .tokenexception import TokenException
 
 
 DEFAULT_RESPONSE_MODES: dict[str, str] = {
@@ -56,6 +59,7 @@ class AuthorizationRequest(pydantic.BaseModel):
     redirect_uri: RedirectURL
     scope: set[str]
     state: str | None
+    access_type: AccessType
 
     # The Response Mode determines how the Authorization Server returns
     # result parameters from the Authorization Endpoint. Non-default
@@ -118,6 +122,7 @@ class AuthorizationRequest(pydantic.BaseModel):
             redirect_uri=params.get_redirect_url(),
             scope=params.get_scope(),
             state=params.state,
+            access_type=params.access_type,
             response_mode=params.response_mode,
             resource=params.get_resources(),
             nonce=params.nonce,
@@ -129,7 +134,7 @@ class AuthorizationRequest(pydantic.BaseModel):
             id_token_hint=params.id_token_hint,
             login_hint=params.login_hint,
             acr_values=params.get_acr_values(),
-            amr_values=params.get_amr_values(),
+            amr_values=params.get_amr_values(), # type: ignore
             registration=params.registration,
             claims=params.get_claims(),
             access_token=None,
@@ -155,6 +160,10 @@ class AuthorizationRequest(pydantic.BaseModel):
     #    assert self.redirect_uri is not None # nosec
     #    return self.redirect_uri.authorize(**params)
 
+    def allows_refresh(self) -> bool:
+        """Return a boolean if refresh tokens are requested."""
+        return self.access_type == AccessType.offline
+
     def can_interact(self) -> bool:
         """Return a boolean indicating if the authorization server or its
         delegated can interact with the resource owner.
@@ -223,16 +232,18 @@ class AuthorizationRequest(pydantic.BaseModel):
         # in Section 4.1.1, and their values MUST be identical."
         # (RFC 6749, 4.1.3)
         if self.redirect_uri_provided and not grant.redirect_uri:
-            raise InvalidGrant(
-                error_description=(
+            raise TokenException(
+                code='invalid_grant',
+                description=(
                     "The authorization request specified the \"redirect_uri\" "
                     "parameter, thus the token request must include the "
                     "same parameter with an identical value."
                 )
             )
         if str(self.redirect_uri) != grant.redirect_uri:
-            raise InvalidGrant(
-                error_description=(
+            raise TokenException(
+                code='invalid_grant',
+                description=(
                     "The \"redirect_uri\" parameter must be identical to the "
                     "value provided in the authorization request."
                 )
@@ -278,4 +289,10 @@ class AuthorizationRequest(pydantic.BaseModel):
         """
         if isinstance(value, str):
             value = {value}
-        return bool(value & set(self.amr_values))
\ No newline at end of file
+        return bool(value & set(self.amr_values))
+
+    def wants_refresh_token(self) -> bool:
+        """Return a boolean indicating if the authorization request
+        wants a refresh token.
+        """
+        return self.access_type == AccessType.offline
\ No newline at end of file
