@@ -1,4 +1,5 @@
 """Declares :class:`IClient`."""
+import datetime
 from typing import cast
 from typing import Any
 from typing import Union
@@ -145,6 +146,16 @@ class IClient(pydantic.BaseModel):
     ) -> RedirectURL | None:
         raise NotImplementedError
 
+    def get_refresh_token_expires(self, now: int) -> int:
+        """Return a :class:`datetime.datetime` instance indicating the
+        Time-To-Live (TTL) of a refresh token.
+        """
+        raise NotImplementedError
+
+    def get_sector_identifier(self) -> Any:
+        """Return the sector identifier used by this client."""
+        raise NotImplementedError
+
     def get_subject_id(self, subject: ISubject) -> int:
         """Return the subject identifier to add to an access token or ID
         token.
@@ -171,6 +182,17 @@ class IClient(pydantic.BaseModel):
         """Return a boolean indicating if this client assigns a PPID."""
         raise NotImplementedError
 
+    async def create_refresh_token(
+        self,
+        codec: ckms.jose.PayloadCodec,
+        subject: ISubject,
+        authorization_id: int,
+        token_id: int,
+        expires: datetime.datetime | None = None
+    ) -> str:
+        """Create a new refresh token using the client parameters."""
+        raise NotImplementedError
+
     async def issue_token(
         self,
         *,
