@@ -7,8 +7,12 @@ from ckms.types import ClaimSet
 
 from ..exceptions import Error
 from .authorizationcode import AuthorizationCode
+from .authorizationidentifier import AuthorizationIdentifier
 from .authorizationrequest import AuthorizationRequest
 from .baseauthorizationrequest import BaseAuthorizationRequest
+from .iauthorization import IAuthorization
+from .irefreshtoken import IRefreshToken
+from .refreshtokenidentifier import RefreshTokenIdentifier
 
 
 class IStorage:
@@ -32,15 +36,37 @@ class IStorage:
         """Delete the provided authorization request"""
         raise NotImplementedError
 
+    @functools.singledispatchmethod
     async def get(
         self,
-        key: str
-    ) -> typing.Optional[typing.Union[bytes, str]]:
+        key: AuthorizationIdentifier | RefreshTokenIdentifier
+    ) -> Any:
         """Return a key from the transient storage backend, or ``None``
         if it does not exist.
         """
         raise NotImplementedError
 
+    @get.register
+    async def _get_authorization_by_natural_key(
+        self,
+        key: AuthorizationIdentifier
+    ) -> Any:
+        return await self.get_authorization_by_natural_key(key.client_id, key.sub)
+
+    @get.register
+    async def _get_refresh_token_by_id(
+        self,
+        key: RefreshTokenIdentifier
+    ) -> IRefreshToken | None:
+        return await self.get_refresh_token_by_id(key)
+
+    async def get_authorization_by_natural_key(
+        self,
+        client_id: int | str,
+        subject_id: int | str
+    ) -> Any:
+        raise NotImplementedError
+
     async def get_authorization_request(
         self,
         request_id: str
@@ -61,6 +87,15 @@ class IStorage:
         """
         raise NotImplementedError
 
+    async def get_refresh_token_by_id(
+        self,
+        key: RefreshTokenIdentifier
+    ) -> IRefreshToken | None:
+        """Return the :class:`~cbra.ext.oauth2.types.IRefreshToken` by its
+        identifier `key`.
+        """
+        raise NotImplementedError
+
     async def set(
         self,
         key: str,
@@ -80,11 +115,18 @@ class IStorage:
     @functools.singledispatchmethod
     async def persist(
         self,
-        obj: AuthorizationRequest | AuthorizationCode
+        obj: AuthorizationRequest | AuthorizationCode | IAuthorization | IRefreshToken
     ) -> None:
         """Persist an object to the transient storage."""
         raise NotImplementedError(type(obj).__name__)
 
+    @persist.register
+    async def _persist_authorization(
+        self,
+        obj: IAuthorization
+    ) -> None:
+        return await self.persist_authorization(obj)
+
     @persist.register
     async def _persist_authorizationcode(
         self,
@@ -99,6 +141,19 @@ class IStorage:
     ) -> None:
         return await self.persist_authorizationrequest(obj)
 
+    @persist.register
+    async def _persist_refresh_token(
+        self,
+        obj: IRefreshToken
+    ) -> None:
+        return await self.persist_refresh_token(obj)
+
+    async def persist_authorization(
+        self,
+        obj: IAuthorization
+    ) -> None:
+        raise NotImplementedError
+
     async def persist_authorizationcode(
         self,
         obj: AuthorizationCode
@@ -117,6 +172,15 @@ class IStorage:
         """
         raise NotImplementedError
 
+    async def persist_refresh_token(
+        self,
+        obj: IRefreshToken
+    ) -> None:
+        """Persist an :class:`cbra.ext.oauth2.types.IRefreshToken`
+        instance.
+        """
+        raise NotImplementedError
+
     async def pop(
         self,
         key: str
