@@ -24,12 +24,20 @@ class ISubject:
     #: Used for testing purposes only.
     #client_scope: defaultdict[str, set[str]]
 
-    def allows_scope(self, scope: set[str]) -> bool:
+    def allows_scope(self, scope: Any) -> bool:
         """Return a boolean indicating that the requested scope `scope`
         was allowed by the subject.
         """
         raise NotImplementedError
 
+    def get_claim(self, name: str) -> Any:
+        """Return the specified OIDC claim for this subject."""
+        raise NotImplementedError
+
+    def get_identifier(self, public: bool = False) -> Any:
+        """Return the identifier for this Subject."""
+        raise NotImplementedError
+
     def register_public_key(self, key: JSONWebKey) -> None:
         """Register a public key for the :class:`ISubject`."""
         raise NotImplementedError
