@@ -1,11 +1,13 @@
 # pylint: skip-file
 from .default import DefaultContentNegotiation
+from .json import JSONDefaultContentNegotiation
 from .null import NullContentNegotiation
 from .nullresponse import NullResponseContentNegotiation
 
 
 __all__ = [
     'DefaultContentNegotiation',
+    'JSONDefaultContentNegotiation',
     'NullContentNegotiation',
     'NullResponseContentNegotiation',
 ]
