@@ -3,6 +3,8 @@
 # All rights reserved. No warranty, explicit or implicit, provided. In
 # no event shall the author(s) be liable for any claim or damages.
 """Declares :class:`IPrincipal`."""
+from typing import Any
+
 from ckms.types import ClaimSet
 
 
@@ -25,9 +27,19 @@ class IPrincipal:
         """
         raise NotImplementedError
 
-    def is_authenticated(self) -> bool:
-        """Return a boolean indicating if the principal is authenticated. Most
-        commonly this will imply that :attr:`sub` is not ``None``.
+    def get_auth_time(self) -> int:
+        """Return an integer indicating the date/time at which the
+        principal last authenticated, in seconds since the UNIX epoch.
+        """
+        raise NotImplementedError
+
+    def get_current_scope(self) -> set[str]:
+        """Return the scope that is currrently granted to the principal."""
+        raise NotImplementedError
+
+    def get_session_claims(self) -> dict[str, Any]:
+        """Return a dictionary holding claims about the current
+        session that is established by the principal.
         """
         raise NotImplementedError
 
@@ -40,5 +52,11 @@ class IPrincipal:
         """
         raise NotImplementedError
 
+    def is_authenticated(self) -> bool:
+        """Return a boolean indicating if the principal is authenticated. Most
+        commonly this will imply that :attr:`sub` is not ``None``.
+        """
+        raise NotImplementedError
+
     def __bool__(self) -> bool:
         return self.is_authenticated()
\ No newline at end of file
